/* 
Libsynth - A signal synthesis library	
Copyright (C) 2018 Ryan Medeiros

    Libsynth is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Libsynth is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libsynth.  If not, see <https://www.gnu.org/licenses/>.
    
You may contact me at lofenyy (at) linuxmail (dot) org. 
*/

/* Index:
	1) Basics
		1.1) Oscillator
		1.2) Noise
		1.3) Sawtooth
	2) Wave filters
		2.1) Sine
		2.2) Pulse/Square
		2.3) Triangle
	3) Utilities
		3.1) Amplifier (planned)
		3.2) Mixer (planned)
		3.3) Envelope Generator
	4) Frequency filters (planned)
*/

#include<stdlib.h>
#include<math.h>

/* 1) Basics
	1.1) Oscillator
	1.2) Noise

The oscillator and noise generator function as signal sources. They are
generally to be used as the foundation that projects are built upon. They may
also be used creatively to create interesting effects, such as adding a tremolo
or vibrato effect. 

Example:
*/

double synth_Oscil(double increment, double shape, double pitch, double sample_rate);
double synth_Noise();

/*	1.3) Sawtooth
The sawtooth wave seems far too complex to be implemented through a filter.
I've decided to implement it as a source, and later via Fourier synthesis later
on. 
*/

double synth_Sawtooth(double increment, double pitch, double sample_rate);

/* 2) Wave filters
Each of the basic wave forms commonly used in synthesis can be acheived by
plugging an unmodified signal source, such as the oscillator, into a shape
function. This is typically computationally cheaper than Fourier methods.

	2.1) Sine
The sound returned from the oscillator function is a pure sine wave with an
amplitude of 1 and a median of 0. There's no need for a shape function.
	
	2.2) Pulse/Square
To adjust the pulse with, adjust the shape parameter on the oscillator. 
*/

double synth_Square(double input);

//	2.3) Triangle

double synth_Triangle(double input);

// 3) Utilities
//	3.3) Envelope Generator

double synth_Env(double increment, double starttime, double startvol, double stopvol, double duration, double sample_rate);

double note2freq(char* note, int octave);

#include"libsynth.c"
